local Giraffe = {}
Giraffe.__index = Giraffe

local ANGLIM	= math.pi/7
local FIRST		= 0.15
local FIRSTLIM	= math.pi/6

local function drawPolyBody( poly, shape )
	local body	= shape and poly or poly.body
	shape		= shape or poly.shape
	love.graphics.polygon("fill", body:getWorldPoints(shape:getPoints()))
end

function Giraffe.new( x, y, keys, xs )
	local self = setmetatable({}, Giraffe)
	
	xs = xs or 1
	self.xs = xs
	
	self.body			= {}
	self.body.body		= love.physics.newBody( world, x, y, "dynamic" )
	self.body.shapes	= { love.physics.newPolygonShape(	-100 * xs, -35,
															 -80 * xs, -55,
															  80 * xs, -55,
															 100 * xs, -30,
															 100 * xs,  40,
															-100 * xs,  40 ),
							love.physics.newPolygonShape(	-100 * xs,  40,
															 -70 * xs,  40,
															 -70 * xs,  90,
															-100 * xs,  90 ),
							love.physics.newPolygonShape(	 100 * xs,  40,
															  70 * xs,  40,
															  70 * xs,  90,
															 100 * xs,  90 ) } 
	self.body.fixtures	= {	love.physics.newFixture( self.body.body, self.body.shapes[1], 5 ),
							love.physics.newFixture( self.body.body, self.body.shapes[2], 5 ),
							love.physics.newFixture( self.body.body, self.body.shapes[3], 5 ) }
	each( self.body.fixtures, function ( fix ) fix:setFriction( 0.7 ) end, 2 )
	
	self:buildNeck( x+70*xs, y-25, 6, 50, -2, -30, 2, 300000 )
	
	self.neck[1].body:setAngle( FIRST*xs )
	self.body.neckJoint = love.physics.newRevoluteJoint( self.body.body, self.neck[1].body, x+70*xs, y-30 )
	self.body.neckJoint:setLimits( -FIRSTLIM+FIRST*xs, FIRSTLIM+FIRST*xs )
	
	self.body.neckJoint:setLimitsEnabled( true )
	self.body.neckJoint:setMotorEnabled( true )
	
	local ln = self.neck[#self.neck].body
	self.head = {}
	self.head.body		= love.physics.newBody( world, ln:getX()+20*xs, ln:getY() - 40, "dynamic" )
	self.head.shapes	= { love.physics.newRectangleShape( 70, 30 ),
							love.physics.newRectangleShape( 39*xs, 0, 8, 29 ) }
	self.head.fixtures	= { love.physics.newFixture( self.head.body, self.head.shapes[1] ),
							love.physics.newFixture( self.head.body, self.head.shapes[2], 0 ) }
	self.head.joint		= love.physics.newWeldJoint( self.head.body, ln, ln:getX()+15*xs, ln:getY()-40 )
	self.head.fixtures[2]:setSensor( true )
	self.head.fixtures[2]:setCategory( CAT_GRAB )
	self.head.body:setUserData( self )
	
	--self.tail = {}
	
	self.stencil = function ()		
		for _,seg in ipairs(self.neck) do
			drawPolyBody( seg )
		end
		for i=1,3 do
			drawPolyBody( self.body.body, self.body.shapes[i] )
		end
		drawPolyBody( self.head.body, self.head.shapes[1] )
	end
	
	self.keys			= keys
	self.couldGrab		= {}
	self.ix, self.iy	= 0, 0
	
	return self
end

function Giraffe:destroy()
	self.body.body:destroy()
	self.head.body:destroy()
	for _,seg in ipairs(self.neck) do
		seg.body:destroy()
	end
end

function Giraffe:draw( ... )
	self.ix, self.iy = drawWithRelStencil( Image.giraffe, self.stencil, 256, self.ix, self.iy, ... )
end

function Giraffe:update( dt )
	if love.keyboard.isDown( self.keys.movel ) then
		self.body.body:applyLinearImpulse( -400, 0 )
	end
	if love.keyboard.isDown( self.keys.mover ) then
		self.body.body:applyLinearImpulse( 400, 0 )
	end
	
	if love.keyboard.isDown( self.keys.grab ) then
		if #self.couldGrab > 0 and not self.head.bitejoint then
			self.head.bitejoint = love.physics.newWeldJoint( self.head.body, self.couldGrab[1]:getBody(), self.head.body:getWorldPoint( 0, 35 ) )
		end
	else
		if self.head.bitejoint then
			self.head.bitejoint:destroy()
			self.head.bitejoint = nil
		end
	end
	
	local angle = (self.body.neckJoint:getJointAngle()-FIRST) / FIRSTLIM
	local speed, torque = -angle, 0
	if love.keyboard.isDown( self.keys.tiltl ) then
		speed	= speed - 2
		torque	= -1 / math.abs(angle)
	end
	if love.keyboard.isDown( self.keys.tiltr ) then
		speed = speed + 2
		torque	= torque + 1 / math.abs(angle)
	end
	torque = 100000 * math.abs( torque + 200*angle )
	self.body.neckJoint:setMaxMotorTorque( torque )
	self.body.neckJoint:setMotorSpeed( speed )
	
	for i=2,#self.neck do
		local seg = self.neck[i]
		seg.joint:setMaxMotorTorque( seg.mult * 5 * math.pow(seg.joint:getJointAngle() / ANGLIM,2) )
		seg.joint:setMotorSpeed( -seg.joint:getJointAngle() / ANGLIM - (speed) )
	end
	
	self.ix = self.ix - 6 * dt
	self.iy = self.iy - 7 * dt
	
	self.couldGrab = {}
end

function Giraffe:buildNeck( x, y, num, sw, ew, h, dh, sm )
	local w, step = sw/2, ew --(sw-ew)/num/2
	local dm = -1 * sm/(num+6)
	self.neck = {}

	for i=1,num do
		local seg = {}
		seg.body	= love.physics.newBody( world, x, y+h/2, "dynamic" )
		seg.shape	= love.physics.newPolygonShape( -w, -h*0.2, -(w+step), h*1.2, w+step, h*1.2, w, -h*0.2 )
		seg.fixture	= love.physics.newFixture( seg.body, seg.shape )
		
		if i > 1 then
			seg.joint = love.physics.newRevoluteJoint( seg.body, self.neck[i-1].body, x, y+h/2 )
			seg.joint:setLimits( -ANGLIM, ANGLIM )
			seg.joint:setLimitsEnabled( true )
			seg.joint:setMotorEnabled( true )
			seg.mult = sm
			sm = sm + dm
		end
		
		self.neck[i] = seg
		
		y, w = y+h, w + step
		h = h - dh
	end
end

return Giraffe