width, height = love.graphics.getDimensions()

function inn( a, ... )
	local t = {...}
	for _,v in ipairs( t ) do
		if a == v then return true end
	end
	return false
end

function swap( a, ... )
	local r = {...}
	r[#r+1] = a
	return unpack(r)
end

function each( tbl, f, s, e )
	local snd = {}
	if e then
		s = s or 1
	elseif s then
		if e == false then
			e = #tbl
		else
			e = s
			s = 1
		end	
	else
		s = 1
		e = #tbl
	end
	for i=s,e do
		snd[i] = f( tbl[i], i )
	end
	return snd
end

local function proxy( f, d )
	return setmetatable({stor={}}, {
		__index = function ( t, k )
			local v = f(k)
			t[k] = v
			return v
		end,
		__call = function ( t, k )
			t[k] = nil
			if d then d( k ) end
		end
	})
end

State = proxy(function ( name )
	return require( 'states.' .. name )
end,
function ( name )
	package.loaded['states.'..name] = nil
end)

Image = proxy(function ( name )
	local i = love.graphics.newImage( 'assets/images/' .. name .. '.png' )
	i:setFilter( "nearest", "nearest" )
	i:setWrap( "repeat", "repeat" )
	return i
end)

local volumes = { hitmarker = 0.8 }

Sound = proxy(function ( var )
	local s = love.audio.newSource( 'assets/audio/'..var..'.wav', 'static' )
	s:setVolume( volumes[var] or 1 )
	return s
end)

function circleStencil( x, y, r )
	return function ()
		love.graphics.circle( "fill", x, y, r, 200 )
	end
end

function drawWithRelStencil( image, stencil, scale, x, y, color, prescale, quad )
	local w, h = image:getDimensions()
	x, y = 0, 0
	scale = scale or 40
	
	love.graphics.push()
	love.graphics.setColor( color or {255,255,255} )
	love.graphics.scale( prescale or 1 )
	love.graphics.setStencil( stencil )
	love.graphics.push()
	love.graphics.origin()
	love.graphics.draw( image, quad or util.screenquad, (x or 0)-w*scale, (y or 0)-h*scale, 0, scale, scale )
	love.graphics.pop()
	love.graphics.setStencil()
	love.graphics.pop()
	
	if x > w*scale then x = x - (w*scale) end
	if y > w*scale then y = y - (h*scale) end
	return x, y
end

return {
	screenquad = love.graphics.newQuad( 0, 0, width, height, 1, 1 )
}
