local Basketball = {}

local Giraffe	= require"giraffe"
local hitmarker	= require"hitmarker"
local debugDraw	= require"debugdraw"

local score
local balls, giraffe, ground, basket
local ballx, bally

local ballStencil, drawObjects, spawnBall, beginContact

CAT_SCOREABLE, CAT_GOAL, CAT_GRAB = 2, 3, 4

--GAMESTATE-HOOKS-------------

function Basketball:enter()
	score = 0
	balls = {}
	ballx, bally = 0, 0
	
	love.graphics.setBackgroundColor( 230, 230, 255 )
	shaders.rainbow:send( "time", love.timer.getTime() )
	shadowCanvas = love.graphics.newCanvas( width, height )
	
	world:setCallbacks( beginContact )
	
	ground = {}
	ground.body		= love.physics.newBody( world, width, height )
	ground.shapes	= {
		love.physics.newRectangleShape( 0, -height, width*2, 20 ), -- top
		love.physics.newRectangleShape(  width, 0, 20, height*2 ), -- right
		love.physics.newRectangleShape( 0,  height, width*2, 20 ), -- bottom
		love.physics.newRectangleShape( -width, 0, 20, height*2 ), -- left
	}
	ground.fixtures = each( ground.shapes, function ( shape ) return love.physics.newFixture( ground.body, shape ) end )
	
	
	giraffe = Giraffe.new( width-100, height+400, { movel="a", mover="d", grab="lshift", tiltl="g", tiltr="h" } )
	
	basket = {}
	basket.body		= love.physics.newBody( world, width*2-400, height )
	basket.shapes	= {
		love.physics.newRectangleShape( 180, 5 ),
		love.physics.newPolygonShape( -110, -10, -110, 10, -90, 10, -95, 8, -95, -10 ),
		love.physics.newPolygonShape(  110, -10,  110, 10,  90, 10,  95, 8,  95, -10 ),
		love.physics.newRectangleShape( 112, -50, 15, 200 ),
	}
	basket.fixtures = each( basket.shapes, function (shape) return love.physics.newFixture( basket.body, shape ) end )
	basket.fixtures[1]:setCategory( CAT_GOAL )
	basket.fixtures[1]:setSensor( true )
	
	spawnBall()
	
	m = 2
end

function Basketball:leave()
	world:setCallbacks( nil, nil, nil, nil )
	for _,bdy in ipairs( world:getBodyList() ) do
		bdy:destroy()
	end
end

function Basketball:draw( second )
	--BACKGROUND------------------
	if love.graphics.isSupported( "shader" ) then
		love.graphics.setShader( shaders.rainbow )
		love.graphics.rectangle( "fill", 0, 0, width, height )
		love.graphics.setShader()
	end
	
	if false then --love.keyboard.isDown"q" then
		love.graphics.scale( 0.5 )
		debugDraw( world, 0, 0, width*2, height*2 )
		return
	end
	
	--SHADOWS---------------------
	if second then love.graphics.setCanvas( shadowCanvas, second )
	else love.graphics.setCanvas( shadowCanvas ) end
	love.graphics.push()
	
	-- fade old shadows
	love.graphics.setColor( 255, 255, 255, 210 )
	love.graphics.setBlendMode( "multiplicative" )
	love.graphics.rectangle("fill", 0, 0, width, height)
	
	-- build new ones
	love.graphics.setBlendMode( "alpha" )
	love.graphics.scale( 0.5 )
	
	if m == 1 then
		drawObjects( {0,0,0,100}, 1.03 )
	elseif m == 2 then
		drawObjects( {140,140,140,140}, 1.01 )
	elseif m == 3 then
		drawObjects( {80,80,80,40} )
	elseif m == 4 then
		drawObjects( {255,80,80,125} )
	else
		drawObjects( {0,0,0,100} )
	end
	
	love.graphics.pop()
	if second then love.graphics.setCanvas( second )
	else love.graphics.setCanvas() end
	love.graphics.setBlendMode( "alpha" )
	
	-- draw on screen
	if love.graphics.isSupported( "shader" ) then
		love.graphics.setShader( shaders.blur )
	end
	love.graphics.draw( shadowCanvas )
	love.graphics.setShader()
	
	--FOREGROUND------------------
	love.graphics.push()
	love.graphics.scale( 0.5 )
	love.graphics.setColor( 193, 47, 14 )
	love.graphics.setLineWidth( 15 )
	love.graphics.setLineJoin( "bevel" )
	love.graphics.line( width*2-300, height, width*2-150, height+50, width*2-150, height*2 )
	each( basket.shapes, function (shape) love.graphics.polygon( "fill", basket.body:getWorldPoints(shape:getPoints()) ) end, 2, false )
	each( ground.shapes, function (shape) love.graphics.polygon( "fill", ground.body:getWorldPoints(shape:getPoints()) ) end )
	drawObjects()
	
	hitmarker.draw()
	love.graphics.pop()
	
	love.graphics.printf( ""..score, width/2+280, height/2-70, 30, "left", 0, 5, 5 )
end

function Basketball:update( dt )
	giraffe:update( dt )
	
	ballx = ballx + 15 * dt
	bally = bally + 10 * dt
	
	shaders.rainbow:send( "time", love.timer.getTime() )
	
	flux.update( dt )
	world:update( math.min(dt,0.2) )
end

function Basketball:keypressed( key )
	if key == "b" then
		spawnBall()
	elseif key == "r" then
		giraffe:destroy()
		giraffe = Giraffe.new( width-100, height+400, { movel="a", mover="d", grab="lshift", tiltl="g", tiltr="h" } )
	elseif key == "m" then
		m = m+1
		if m > 5 then m = 1 end
	end
end

--SMALL-ROUTINES-------------

function ballStencil()
	for _,ball in ipairs( balls ) do
		love.graphics.circle("fill", swap(20, ball.body:getWorldCenter()) )
	end
end

function drawObjects( ... )
	ballx, bally = drawWithRelStencil( Image.ball, ballStencil, 80, ballx, bally, ... )
	giraffe:draw( ... )
end

function spawnBall()
	local ball = {}
	ball.body		= love.physics.newBody( world, width*1.3, height, "dynamic" )
	ball.shape		= love.physics.newCircleShape( 20 )
	ball.fixture	= love.physics.newFixture( ball.body, ball.shape, 1 )
	ball.fixture:setRestitution( 0.7 )
	ball.fixture:setCategory( CAT_SCOREABLE )
	ball.body:setUserData( ball )
	ball.body:setBullet( true )
	
	ball.body:setLinearVelocity( math.random(-50, 50), math.random(-50, 50) )

	balls[#balls+1] = ball
end


--COLLISION------------------

function beginContact( a, b, col )
	local mobile, touch, grab, grabbed
	if inn( CAT_SCOREABLE, a:getCategory() ) then
		mobile = a
	elseif inn( CAT_GOAL, a:getCategory() ) then
		touch = a
	end
	if inn( CAT_GRAB, a:getCategory() )  then
		grab	= a
		grabbed	= b
	end
	
	if inn( CAT_SCOREABLE, b:getCategory() ) then
		mobile = b
	elseif inn( CAT_GOAL, b:getCategory() ) then
		touch = b
	end
	if inn( CAT_GRAB, b:getCategory() ) then
		grab	= b
		grabbed	= a
	end
	
	if grab then
		table.insert( grab:getBody():getUserData().couldGrab, grabbed )
		return
	elseif mobile and touch then
		score = score + 1
		mobile = mobile:getBody():getUserData()
		hitmarker.many( mobile.body:getWorldCenter() )
		hitmarker.new( giraffe.head.body:getWorldCenter() )
		mobile.justScored = true
		return
	elseif inn( a:getCategory(), CAT_SCOREABLE ) or inn( b:getCategory(), CAT_SCOREABLE ) then
		Sound["hit"..math.random(1,4)]:play()
	end
end

return Basketball
