local menu = {}

local pos, stencil

local hand = love.mouse.getSystemCursor( "hand" )
local basket, volley

local help = true

function menu:enter( from, ... )
	love.graphics.setCanvas( lastState )
	from:draw( lastState )
	love.graphics.setCanvas()
	
	local x, y = width/2, height/2
	pos = { x=x,y=-400,a=0 }
	flux.to( pos, 0.8, {y=y-200,a=100} )
	
	stencil = function ( x, y )
		return function ()
			love.graphics.setShader( shaders.mask )
	
			love.graphics.draw( Image.logo, pos.x+x-150, pos.y+y )
			love.graphics.draw( Image.basketball, pos.x+x, pos.y+y+200, 0, 1.5, 1.5, 100, 0 )
			love.graphics.draw( Image.volleyball, pos.x+x, pos.y+y+270, 0, 1.5, 1.5, 100, 0 )
		
			love.graphics.setShader()
		end
	end
	basket = function ( x, y )
		return x >= pos.x-160 and x <= pos.x+150 and y >= pos.y+190 and y <= pos.y+240
	end
	volley = function ( x, y )
		return x >= pos.x-160 and x <= pos.x+150 and y >= pos.y+260 and y <= pos.y+310
	end
end

function menu:draw()
	love.graphics.setColor( 255, 255, 255, 255 )
	love.graphics.draw( lastState )
	
	if help then
		each( {"a/d", "g/h", "shift", "esc", "b", "r", "f", "m", "space", "h"}, function (str, i) love.graphics.print( str, 7, 15*i-10 ) end )
		each( {"left/right", "np4/np6", "np5"}, function (str, i) love.graphics.print( str, 50, 15*i-10 ) end )
		each( {"walk", "tilt neck", "bite (grab)", "pause/quit", "spawn Ball", "reset Position", "fullscreen", "mute music", "resume game", "hide this message"}, function (str, i)
			love.graphics.print( "- "..str, 120, 15*i-10 ) end )
		love.graphics.print( "music: 'HyperSpaceAdventure' by Skip", 557, 580 )
	end
	
	love.graphics.setColor( 44, 107, 19, 255 )
	love.graphics.setStencil( stencil(-3,3) )
	love.graphics.rectangle("fill", 0, 0, width, height )
	
	love.graphics.setColor( 255, 255, 255, 255 )
	love.graphics.setStencil( stencil(0,0) )
	love.graphics.draw( Image.giraffe, util.screenquad, -100, 0, 0, 256, 256 )
	
	love.graphics.setStencil()
	
	if volley( love.mouse.getPosition() ) or basket( love.mouse.getPosition() ) then
		love.mouse.setCursor( hand )
	else
		love.mouse.setCursor()
	end
	
end

function menu:leave()
	love.mouse.setCursor()
end

function menu:mousepressed( x, y, key )
	if volley( x, y ) then
		Gamestate.pop()
		Gamestate.switch( State.volleyball )
	elseif basket( x, y ) then
		Gamestate.pop()
		Gamestate.switch( State.basketball )
	end
end

function menu:keypressed( key )
	if key == " " then
		Gamestate.pop()
	elseif key == "h" then
		help = not help
	end
end

return menu