local Volleyball = {}

local Giraffe	= require"giraffe"
local hitmarker	= require"hitmarker"
local debugDraw	= require"debugdraw"

local score
local balls, players, ground, net
local ballx, bally

local ballStencil, drawObjects, spawnBall, beginContact

CAT_SCOREABLE, CAT_GOAL, CAT_GRAB = 2, 3, 4

--GAMESTATE-HOOKS-------------

function Volleyball:enter()
	score = {0,0}
	balls = {}
	ballx, bally = 0, 0
	
	love.graphics.setBackgroundColor( 230, 230, 255 )
	shaders.rainbow:send( "time", love.timer.getTime() )
	shadowCanvas = love.graphics.newCanvas( width, height )
	
	world:setCallbacks( beginContact )
	
	ground = {}
	ground.body		= love.physics.newBody( world, width, height )
	ground.shapes	= {
		love.physics.newRectangleShape( 0, -height, width*2, 20 ), -- top
		love.physics.newRectangleShape(  width, 0, 20, height*2 ), -- right
		love.physics.newRectangleShape( 0,  height, width*2, 20 ), -- bottom
		love.physics.newRectangleShape( -width, 0, 20, height*2 ), -- left
	}
	ground.fixtures = each( ground.shapes, function ( shape ) return love.physics.newFixture( ground.body, shape ) end )

	players = {
		Giraffe.new( width+400, height+460, { movel="left", mover="right", grab="kp5", tiltl="kp4", tiltr="kp6" }, -1 ),
		Giraffe.new( width-400, height+460, { movel="a", mover="d", grab="lshift", tiltl="g", tiltr="h" } )
	}
	
	net = {}
	net.body	= love.physics.newBody( world, width, height*2-200 )
	net.shape	= love.physics.newRectangleShape( 10, 500 )
	net.fixture	= love.physics.newFixture( net.body, net.shape )
	
	ground.fixtures[5] = love.physics.newFixture( ground.body, love.physics.newRectangleShape(-width/2, height-20, width, 20 ) )
	ground.fixtures[6] = love.physics.newFixture( ground.body, love.physics.newRectangleShape( width/2, height-20, width, 20 ) )
	each( ground.fixtures, function (shape)
		shape:setSensor( true )
		shape:setCategory( CAT_GOAL )
	end, 5, 6 )
	ground.fixtures[5]:setUserData( 2 )
	ground.fixtures[6]:setUserData( 1 )
	
	spawnBall()
	
	m = 2
end

function Volleyball:leave()
	world:setCallbacks( nil, nil, nil, nil )
	for _,bdy in ipairs( world:getBodyList() ) do
		bdy:destroy()
	end
end

function Volleyball:draw( second )
	--BACKGROUND------------------
	if love.graphics.isSupported( "shader" ) then
		love.graphics.setShader( shaders.rainbow )
		love.graphics.rectangle( "fill", 0, 0, width, height )
		love.graphics.setShader()
	end
	
	if love.keyboard.isDown"q" then
		love.graphics.scale( 0.5 )
		debugDraw( world, 0, 0, width*2, height*2 )
		return
	end
	
	--SHADOWS---------------------
	if second then love.graphics.setCanvas( shadowCanvas, second )
	else love.graphics.setCanvas( shadowCanvas ) end
	love.graphics.push()
	
	-- fade old shadows
	love.graphics.setColor( 255, 255, 255, 210 )
	love.graphics.setBlendMode( "multiplicative" )
	love.graphics.rectangle("fill", 0, 0, width, height)
	
	-- build new ones
	love.graphics.setBlendMode( "alpha" )
	love.graphics.scale( 0.5 )
	
	if m == 1 then
		drawObjects( {0,0,0,100}, 1.03 )
	elseif m == 2 then
		drawObjects( {140,140,140,140}, 1.01 )
	elseif m == 3 then
		drawObjects( {80,80,80,40} )
	elseif m == 4 then
		drawObjects( {255,80,80,125} )
	else
		drawObjects( {0,0,0,100} )
	end
	
	love.graphics.pop()
	if second then love.graphics.setCanvas( second )
	else love.graphics.setCanvas() end
	love.graphics.setBlendMode( "alpha" )
	
	-- draw on screen
	if love.graphics.isSupported( "shader" ) then
		love.graphics.setShader( shaders.blur )
	end
	love.graphics.draw( shadowCanvas )
	love.graphics.setShader()
	
	--FOREGROUND------------------
	love.graphics.push()
	love.graphics.scale( 0.5 )
	love.graphics.setColor( 193, 47, 14 )
	love.graphics.polygon( "fill", net.body:getWorldPoints(net.shape:getPoints()) )
	each( ground.shapes, function (shape) love.graphics.polygon( "fill", ground.body:getWorldPoints(shape:getPoints()) ) end )
	drawObjects()
	
	hitmarker.draw()
	love.graphics.pop()
	
	love.graphics.printf( ""..score[1],	width/2-170, 10, 30, "right", 0, 5, 5 )
	love.graphics.printf( ""..score[2], width/2+20,  10, 30, "left", 0, 5, 5 )
end

function Volleyball:update( dt )
	each( players, function (ply) ply:update( dt ) end)
	
	ballx = ballx + 15 * dt
	bally = bally + 10 * dt
	
	shaders.rainbow:send( "time", love.timer.getTime() )
	
	flux.update( dt )
	world:update( math.min(dt,0.2) )
end

function Volleyball:keypressed( key )
	if key == "b" then
		spawnBall()
	elseif key == "r" then
		each( balls, function (ball) ball.body:destroy() end )
		each( players, Giraffe.destroy )
		balls = {}
		players = {
			Giraffe.new( width+400, height+460, { movel="left", mover="right", grab="kp5", tiltl="kp4", tiltr="kp6" }, -1 ),
			Giraffe.new( width-400, height+460, { movel="a", mover="d", grab="lshift", tiltl="g", tiltr="h" } )
		}
	elseif key == "m" then
		m = m+1
		if m > 5 then m = 1 end
	end
end

--SMALL-ROUTINES-------------

function ballStencil()
	for _,ball in ipairs( balls ) do
		love.graphics.circle("fill", swap(20, ball.body:getWorldCenter()) )
	end
end

function drawObjects( ... )
	ballx, bally = drawWithRelStencil( Image.ball, ballStencil, 80, ballx, bally, ... )
	for _,ply in ipairs( players ) do 
		ply:draw( ... )
	end
end

function spawnBall()
	local ball = {}
	ball.body		= love.physics.newBody( world, width, height-100, "dynamic" )
	ball.shape		= love.physics.newCircleShape( 20 )
	ball.fixture	= love.physics.newFixture( ball.body, ball.shape, 1 )
	ball.fixture:setRestitution( 0.7 )
	ball.fixture:setCategory( CAT_SCOREABLE )
	ball.body:setUserData( ball )
	ball.body:setBullet( true )
	
	ball.body:setLinearVelocity( math.random(-500, 500), math.random(-500, -200) )

	balls[#balls+1] = ball
end
--COLLISION------------------

function beginContact( a, b, col )
	local mobile, touch, grab, grabbed
	if inn( CAT_SCOREABLE, a:getCategory() ) then
		mobile = a
	elseif inn( CAT_GOAL, a:getCategory() ) then
		touch = a
	end
	if inn( CAT_GRAB, a:getCategory() )  then
		grab	= a
		grabbed	= b
	end
	
	if inn( CAT_SCOREABLE, b:getCategory() ) then
		mobile = b
	elseif inn( CAT_GOAL, b:getCategory() ) then
		touch = b
	end
	if inn( CAT_GRAB, b:getCategory() ) then
		grab	= b
		grabbed	= a
	end
	
	if grab then
		table.insert( grab:getBody():getUserData().couldGrab, grabbed )
		return
	end
	
	if mobile and touch then
		mobile:setCategory( 1 )
		mobile = mobile:getBody():getUserData()
		hitmarker.new( mobile.body:getWorldCenter() )
		hitmarker.many( players[touch:getUserData()].body.body:getWorldCenter() )
		score[touch:getUserData()] = score[touch:getUserData()] + 1
		return
	elseif inn( a:getCategory(), CAT_SCOREABLE ) or inn( b:getCategory(), CAT_SCOREABLE ) then
		Sound["hit"..math.random(1,4)]:play()
	end
end


return Volleyball
